# Copyright (c) Kuba Szczodrzyński 2022-05-11.

from ..mixins import HasId, HasVars
from ..utils import Model


class Template(Model, HasId, HasVars):
    title: str
    width: float
    height: float
    front: list[dict]
    back: list[dict] = {}
    pads: dict[str, str] = {}
    test_pads: dict[str, str] = {}

    type: str = "include"
    pos = "0,0"



# Copyright (c) Kuba Szczodrzyński 2022-06-15.

from enum import IntFlag


class PinFeatures(IntFlag):
    PIN_NONE = 1 << 0
    PIN_GPIO = 1 << 1
    PIN_IRQ = 1 << 2
    PIN_PWM = 1 << 3
    PIN_ADC = 1 << 4
    PIN_DAC = 1 << 5
    PIN_I2C = 1 << 6
    PIN_I2S = 1 << 7
    PIN_JTAG = 1 << 8
    PIN_SPI = 1 << 9
    PIN_SWD = 1 << 10
    PIN_UART = 1 << 11



# -*- coding: utf8 -*-

"""vector.py: A simple little V class. Enabling basic vector math. """

__author__ = "Sven Hecht"
__license__ = "GPL"
__version__ = "1.0.1"
__maintainer__ = "Sven Hecht"
__email__ = "info@shdev.de"
__status__ = "Production"

from math import acos, cos, pi, sin, sqrt
from random import choice, random


class V:
    x: float
    y: float

    def __init__(self, x=0, y=0):
        self.x = 0
        self.y = 0
        if isinstance(x, tuple) or isinstance(x, list):
            y = x[1]
            x = x[0]
        elif isinstance(x, V):
            y = x.y
            x = x.x

        self.set(x, y)

    @staticmethod
    def random(size=1):
        sizex = size
        sizey = size
        if isinstance(size, tuple) or isinstance(size, list):
            sizex = size[0]
            sizey = size[1]
        elif isinstance(size, V):
            sizex = size.x
            sizey = size.y
        return V(random() * sizex, random() * sizey)

    @staticmethod
    def randomUnitCircle():
        d = random() * pi
        return V(cos(d) * choice([1, -1]), sin(d) * choice([1, -1]))

    @staticmethod
    def distance(a, b):
        return (a - b).length

    @staticmethod
    def angle(v1, v2):
        return acos(v1.dotproduct(v2) / (v1.length * v2.length))

    @staticmethod
    def angleDeg(v1, v2):
        return V.angle(v1, v2) * 180.0 / pi

    def set(self, x, y):
        self.x = x
        self.y = y

    @property
    def array(self) -> list[float]:
        return [self.x, self.y]

    @property
    def tuple(self) -> tuple[float, float]:
        return (self.x, self.y)

    @property
    def int(self) -> "V":
        return V(int(self.x), int(self.y))

    def getNormalized(self):
        if self.length != 0:
            return self / self.length
        else:
            return V(0, 0)

    def dotproduct(self, other):
        if isinstance(other, V):
            return self.x * other.x + self.y * other.y
        elif isinstance(other, tuple) or isinstance(other, list):
            return self.x * other[0] + self.y * other[1]
        else:
            return NotImplemented

    def __add__(self, other):
        if isinstance(other, V):
            return V(self.x + other.x, self.y + other.y)
        elif isinstance(other, tuple) or isinstance(other, list):
            return V(self.x + other[0], self.y + other[1])
        elif isinstance(other, int) or isinstance(other, float):
            return V(self.x + other, self.y + other)
        else:
            return NotImplemented

    def __sub__(self, other):
        if isinstance(other, V):
            return V(self.x - other.x, self.y - other.y)
        if isinstance(other, tuple) or isinstance(other, list):
            return V(self.x - other[0], self.y - other[1])
        elif isinstance(other, int) or isinstance(other, float):
            return V(self.x - other, self.y - other)
        else:
            return NotImplemented

    def __rsub__(self, other):
        if isinstance(other, V):
            return V(other.x - self.x, other.y - self.y)
        elif isinstance(other, tuple) or isinstance(other, list):
            return V(other[0] - self.x, other[1] - self.y)
        elif isinstance(other, int) or isinstance(other, float):
            return V(other - self.x, other - self.y)
        else:
            return NotImplemented

    def __mul__(self, other):
        if isinstance(other, V):
            return V(self.x * other.x, self.y * other.y)
        elif isinstance(other, tuple) or isinstance(other, list):
            return V(self.x * other[0], self.y * other[1])
        elif isinstance(other, int) or isinstance(other, float):
            return V(self.x * other, self.y * other)
        else:
            return NotImplemented

    def __truediv__(self, other):
        if isinstance(other, V):
            return V(self.x / other.x, self.y / other.y)
        elif isinstance(other, tuple) or isinstance(other, list):
            return V(self.x / other[0], self.y / other[1])
        elif isinstance(other, int) or isinstance(other, float):
            return V(self.x / other, self.y / other)
        else:
            return NotImplemented

    def __rtruediv__(self, other):
        if isinstance(other, V):
            return V(other.x / self.x, other.y / self.y)
        elif isinstance(other, tuple) or isinstance(other, list):
            return V(other[0] / self.x, other[1] / self.y)
        elif isinstance(other, int) or isinstance(other, float):
            return V(other / self.x, other / self.y)
        else:
            return NotImplemented

    def __pow__(self, other):
        if isinstance(other, int) or isinstance(other, float):
            return V(self.x**other, self.y**other)
        else:
            return NotImplemented

    def __iadd__(self, other):
        if isinstance(other, V):
            self.x += other.x
            self.y += other.y
            return self
        elif isinstance(other, tuple) or isinstance(other, list):
            self.x += other[0]
            self.y += other[1]
            return self
        elif isinstance(other, int) or isinstance(other, float):
            self.x += other
            self.y += other
            return self
        else:
            return NotImplemented

    def __isub__(self, other):
        if isinstance(other, V):
            self.x -= other.x
            self.y -= other.y
            return self
        elif isinstance(other, tuple) or isinstance(other, list):
            self.x -= other[0]
            self.y -= other[1]
            return self
        elif isinstance(other, int) or isinstance(other, float):
            self.x -= other
            self.y -= other
            return self
        else:
            return NotImplemented

    def __imul__(self, other):
        if isinstance(other, V):
            self.x *= other.x
            self.y *= other.y
            return self
        elif isinstance(other, tuple) or isinstance(other, list):
            self.x *= other[0]
            self.y *= other[1]
            return self
        elif isinstance(other, int) or isinstance(other, float):
            self.x *= other
            self.y *= other
            return self
        else:
            return NotImplemented

    def __itruediv__(self, other):
        if isinstance(other, V):
            self.x /= other.x
            self.y /= other.y
            return self
        elif isinstance(other, tuple) or isinstance(other, list):
            self.x /= other[0]
            self.y /= other[1]
            return self
        elif isinstance(other, int) or isinstance(other, float):
            self.x /= other
            self.y /= other
            return self
        else:
            return NotImplemented

    def __ipow__(self, other):
        if isinstance(other, int) or isinstance(other, float):
            self.x **= other
            self.y **= other
            return self
        else:
            return NotImplemented

    def __eq__(self, other):
        if isinstance(other, V):
            return self.x == other.x and self.y == other.y
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, V):
            return self.x != other.x or self.y != other.y
        else:
            return NotImplemented

    def __gt__(self, other):
        if isinstance(other, V):
            return self.length > other.length
        else:
            return NotImplemented

    def __ge__(self, other):
        if isinstance(other, V):
            return self.length >= other.length
        else:
            return NotImplemented

    def __lt__(self, other):
        if isinstance(other, V):
            return self.length < other.length
        else:
            return NotImplemented

    def __le__(self, other):
        if isinstance(other, V):
            return self.length <= other.length
        else:
            return NotImplemented

    def __eq__(self, other):
        if isinstance(other, V):
            return self.x == other.x and self.y == other.y
        else:
            return NotImplemented

    def __len__(self):
        return int(sqrt(self.x**2 + self.y**2))

    @property
    def length(self):
        return sqrt(self.x**2 + self.y**2)

    def __getitem__(self, key):
        if key == "x" or key == "X" or key == 0 or key == "0":
            return self.x
        elif key == "y" or key == "Y" or key == 1 or key == "1":
            return self.y

    def __str__(self):
        return "[x: %(x)f, y: %(y)f]" % self

    def __repr__(self):
        return "{'x': %(x)f, 'y': %(y)f}" % self

    def __neg__(self):
        return V(-self.x, -self.y)

    def __bool__(self):
        return True



# Copyright (c) Kuba Szczodrzyński 2022-05-11.

from svgwrite import Drawing

from ..mixins import HasId, HasVars
from ..shapes import Shape, ShapeGroup
from ..utils import Model
from ..vector import V
from .enums import RoleType, Side

PinDict = dict[RoleType, list[str] | str | int | float | None]


class Pcb(Model, HasId, HasVars):
    symbol: str
    templates: list[str]
    scale: float = 0
    ic: dict[int, PinDict] = {}
    front: list[dict] = []
    back: list[dict] = []
    pads: dict[str, str] = {}
    test_pads: dict[str, str] = {}
    pinout: dict[str, PinDict] = {}
    pinout_hidden: str = ""

    shapes: dict[Side, ShapeGroup] = {}

    def get_pos(self, side: Side) -> tuple[V, V]:
        shape = self.shapes[side]
        return (shape.pos1, shape.pos2)

    def get_size(self, side: Side) -> V:
        shape = self.shapes[side]
        return V(shape.width, shape.height)

    def draw(self, dwg: Drawing, side: Side, pos: V):
        shape = self.shapes[side]
        shape.move(pos)
        shape.draw(dwg)
        shape.move(-pos)

    def pad_by_id(self, id: str) -> Shape:
        for side in self.shapes.values():
            shape = side.get_by_id(id)
            if shape:
                return shape
        return None



# Copyright (c) Kuba Szczodrzyński 2022-05-09.

import os
from os.path import join

import click
from click import echo
from devtools import debug
from svgwrite import Drawing, shapes
from svgwrite.utils import AutoID

from . import Core
from .models import Board, Side, Template
from .readme.writer import ReadmeWriter
from .shapes.label import Label
from .utils import load_json
from .variant.writer import VariantWriter
from .vector import V

core = Core()


def load_boards(boards: list[str | Board]) -> list[Board]:
    if not boards:
        return boards
    if isinstance(boards[0], str):
        if boards[0] == "all":
            boards = core.list_json("boards")
        boards = [
            echo(f"Loading board '{board}'...") or core.get_board(board)
            for board in boards
        ]
    return boards


@click.group(help=f"boardgen CLI v{core.version}")
@click.option("--boards", type=str, multiple=True, help="Custom boards directories")
@click.option("--shapes", type=str, multiple=True, help="Custom shapes directories")
@click.option(
    "--templates", type=str, multiple=True, help="Custom templates directories"
)
@click.option("--presets", type=str, multiple=True, help="Custom presets .json")
@click.option("--roles", type=str, multiple=True, help="Custom roles .json")
@click.option("--flash", type=str, multiple=True, help="Custom flash regions .json")
def cli(
    boards: tuple[str],
    shapes: tuple[str],
    templates: tuple[str],
    presets: tuple[str],
    roles: tuple[str],
    flash: tuple[str],
    *args,
    **kwargs,
):
    print(f"boardgen CLI v{core.version}")
    core.add_custom_dirs(
        boards=list(boards),
        shapes=list(shapes),
        templates=list(templates),
    )
    presets_data = {}
    roles_data = {}
    flash_data = {}
    for file in presets:
        presets_data |= load_json(file)
    for file in roles:
        roles_data |= load_json(file)
    for file in flash:
        flash_data |= load_json(file)
    core.add_custom_json(presets=presets_data, roles=roles_data, flash=flash_data)


@cli.command()
@click.argument("boards", nargs=-1, required=True)
@click.option("--dump", "-d", is_flag=True, help="Dump board info and exit")
@click.option("--output", "-o", default=".", help="Output directory")
@click.option("--subdir", "-O", is_flag=True, help="Output into per-board subdirectory")
@click.option("--width", "-w", default=1024, help="Image width (px)")
@click.option("--height", "-h", default=500, help="Image height (px)")
@click.option(
    "--scale",
    "-s",
    default=12,
    help="Diagram size (bigger if scale larger); 0 means automatic",
)
@click.option(
    "--canvas/--no-canvas",
    "-c/-C",
    default=True,
    help="Draw a white background with black border",
)
@click.option("--labels/--no-labels", "-l/-L", default=True, help="Draw pin labels")
@click.pass_context
def draw(
    ctx,
    boards: list[str],
    dump: bool,
    output: str,
    subdir: bool,
    width: int,
    height: int,
    scale: float,
    canvas: bool,
    labels: bool,
):
    """Draw board diagrams"""
    boards = load_boards(boards)
    if dump:
        for board in boards:
            debug(board)
        ctx.exit()

    px_size = V(width, height)

    if output:
        os.makedirs(output, exist_ok=True)

    sides = [Side.FRONT, Side.BACK]

    scale_arg = scale

    for board in boards:
        board: Board
        if not board.pcb or not board.pcb.templates:
            echo(f"Skipping '{board.name}'...")
            continue
        echo(f"Drawing '{board.name}'...")

        pcb = board.pcb
        images = []

        AutoID._set_value(1)
        dwg = Drawing(size=px_size.tuple)

        scale = scale_arg
        if pcb.scale and pcb.scale < scale:
            scale = pcb.scale

        for side in sides:
            try:
                (pcb_pos1, pcb_pos2) = pcb.get_pos(side)
            except ValueError:
                continue
            labels_list = []
            size = pcb_pos2 - pcb_pos1
            pos1 = pcb_pos1
            if labels:
                (labels_list, labels_pos1, labels_pos2) = core.build_labels(pcb, side)
                if labels_list:
                    pos1 = V(
                        min(pcb_pos1.x, labels_pos1.x), min(pcb_pos1.y, labels_pos1.y)
                    )
                    pos2 = V(
                        max(pcb_pos2.x, labels_pos2.x), max(pcb_pos2.y, labels_pos2.y)
                    )
                    size = pos2 - pos1
                else:
                    continue
            images += [
                (side, pos1, size, labels_list),
            ]

        # stack horizontally
        part_size = V(px_size.x / len(images), px_size.y)
        part_pos = [V(part_size.x * i, 0) for i in range(len(images))]

        if scale:
            vb_size = px_size / scale
        else:
            scale = 99999
            size_pad = part_size * 0.90  # 5% padding from each side
            for (_, _, size, _) in images:
                scale = min(scale, size_pad.x / size.x, size_pad.y / size.y)
            echo(" - calculated scale: %.2f" % scale)
            vb_size = px_size / scale

        dwg.viewbox(width=vb_size.x, height=vb_size.y)

        if canvas:
            bg = shapes.Rect(insert=(0, 0), size=vb_size.tuple)
            bg.fill(color="white")
            bg.stroke(color="black", width=0.1)
            dwg.add(bg)

        for i, (side, pos1, size, labels_list) in enumerate(images):
            pcb_pos = ((part_size / scale) - size) / 2
            pcb_pos += part_pos[i] / scale
            pcb_pos -= pos1
            if canvas:
                pcb_pos.x -= 0.05
            pcb.draw(dwg, side, pcb_pos)
            if labels:
                for label in labels_list:
                    label: Label
                    label.move(pcb_pos)
                    label.draw(dwg)
                    label.move(-pcb_pos)

        svg = join(output, f"{board.id}.svg")
        if subdir:
            os.makedirs(join(output, board.id), exist_ok=True)
            svg = join(output, board.id, f"pinout_{board.id}.svg")
        with open(svg, "w", encoding="utf-8") as f:
            dwg.write(f, pretty=True, indent=4)


@cli.command()
@click.argument("boards", nargs=-1, required=True)
@click.option("--output", "-o", default=".", help="Output directory")
@click.option("--subdir", "-O", is_flag=True, help="Output into per-board subdirectory")
def write(
    boards: list[str],
    output: str,
    subdir: bool,
):
    """Write board README.md"""
    boards = load_boards(boards)

    if output:
        os.makedirs(output, exist_ok=True)

    for board in boards:
        board: Board
        readme = ReadmeWriter(core)
        readme.write(board=board)

        md = join(output, f"{board.id}.md")
        if subdir:
            md = join(output, board.id, f"README.md")

        echo(f"Saving to '{md}'...")
        readme.save(md)


@cli.command()
@click.argument("boards", nargs=-1, required=True)
@click.option("--output", "-o", default=".", help="Output directory")
@click.option("--subdir", "-O", is_flag=True, help="Output into per-board subdirectory")
def variant(
    boards: list[str],
    output: str,
    subdir: bool,
):
    """Write board variant definitions (.h/.cpp)"""
    boards = load_boards(boards)

    if output:
        os.makedirs(output, exist_ok=True)

    for board in boards:
        board: Board
        writer = VariantWriter(core)
        writer.generate(board=board)

        out_h = join(output, f"{board.id}.h")
        out_cpp = join(output, f"{board.id}.cpp")
        if subdir:
            out_h = join(output, board.id, f"variant.h")
            out_cpp = join(output, board.id, f"variant.cpp")
            out_pins = join(output, board.id, f"pins_arduino.h")
            writer.save_compat(out_pins)

        echo(f"Saving to '{out_h}' and '{out_cpp}'...")
        board_name = f"{board.id}.json"
        writer.save_h(out_h, board_name)
        if writer.pins:
            writer.save_cpp(out_cpp, board_name)


@cli.command()
@click.argument("boards", nargs=-1, required=True)
@click.option("--output", "-o", default=".", help="Output directory")
@click.option("--subdir", "-O", is_flag=True, help="Output into per-board subdirectory")
@click.pass_context
def all(
    ctx,
    boards: list[str],
    output: str,
    subdir: bool,
):
    """Draw and generate complete board specifications"""
    boards = load_boards(boards)

    ctx.invoke(draw, boards=boards, output=output, subdir=subdir)
    ctx.invoke(write, boards=boards, output=output, subdir=subdir)
    ctx.invoke(variant, boards=boards, output=output, subdir=subdir)


@cli.group(name="list")
def list_cmd():
    """List boards/templates/etc"""


@list_cmd.command(name="boards")
@click.option("--full", "-f", is_flag=True, help="Print more details")
def list_boards(full: bool):
    """List available boards"""
    boards = core.list_json("boards")
    echo("Available boards:")
    for board_name in boards:
        board = core.get_board(board_name)
        echo(f" - '{board_name}': {board.name} / {board.vendor}")
        if full:
            echo(f"    - CPU: {board.build.mcu.upper()} @ {board.cpu_freq}")
            echo(f"    - Flash: {board.size_flash}")
            echo(f"    - RAM: {board.size_ram}")
            echo(f"    - Pin count: {len(board.pcb.pinout)}")
            echo(f"    - Connectivity: {board.connectivity}")


@list_cmd.command(name="templates")
@click.option("--full", "-f", is_flag=True, help="Print more details")
def list_templates(full: bool):
    """List available templates"""
    templates = core.list_json("templates")
    echo("Available templates:")
    for template_name in templates:
        template = Template(**core.load_template(template_name))
        echo(f" - '{template_name}': {template.title}")
        if full:
            echo(f"    - Size: {template.width} x {template.height} mm")
            echo(f"    - Pad count: {len(template.pads)}")


@list_cmd.command(name="shapes")
def list_shapes():
    """List available shapes"""
    templates = core.list_json("shapes")
    echo("Available shapes:")
    for shape_name in templates:
        echo(f" - '{shape_name}'")


@list_cmd.command(name="presets")
def list_presets():
    """List available presets"""
    echo("Available presets:")
    for preset_name, preset in core.presets.items():
        echo(f" - '{preset_name}'")


@list_cmd.command(name="roles")
def list_roles():
    """List available pin roles"""
    echo("Available pin roles:")
    for role_type, role in core.roles.items():
        echo(f" - {role_type.name}: {role.title}")


@list_cmd.command(name="flash")
def list_flash():
    """List available flash regions"""
    echo("Available flash regions:")
    for region_name, description in core.flash.items():
        echo(f" - {region_name}: {description}")



# Copyright (c) Kuba Szczodrzyński 2022-05-12.

from pydantic import Field
from svgwrite import Drawing, shapes

from ..vector import V
from .base import Shape
from .fill_style import FillStyle


class Rect(Shape):
    size_v: V = Field(alias="size")
    rx: float = None
    ry: float = None
    fill: FillStyle = None
    stroke: FillStyle = None

    def draw(self, dwg: Drawing):
        if self.stroke and self.stroke.width:
            self.pos += (self.stroke.width / 2, self.stroke.width / 2)
            self.size_v -= (self.stroke.width, self.stroke.width)
        rect = shapes.Rect(
            insert=self.pos.tuple,
            size=self.size_v.tuple,
            rx=self.rx,
            ry=self.ry,
            id=self.fullid,
        )
        if self.fill:
            self.fill.apply_to(dwg, rect)
        if self.stroke:
            self.stroke.apply_to(dwg, rect, stroke=True)
        dwg.add(rect)

    @property
    def x2(self) -> float:
        return self.x1 + self.size_v.x

    @property
    def y2(self) -> float:
        return self.y1 + self.size_v.y



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['boardgen',
 'boardgen.core',
 'boardgen.models',
 'boardgen.readme',
 'boardgen.shapes',
 'boardgen.shapes.label',
 'boardgen.variant']

package_data = \
{'': ['*'],
 'boardgen': ['res/*', 'res/boards/*', 'res/shapes/*', 'res/templates/*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'devtools>=0.8.0,<0.9.0',
 'pydantic>=1.9.0,<2.0.0',
 'svgwrite>=1.4.2,<2.0.0']

entry_points = \
{'console_scripts': ['boardgen = boardgen.cli:cli']}

setup_kwargs = {
    'name': 'boardgen',
    'version': '0.4.10',
    'description': 'Board pinout diagram generator',
    'long_description': '# boardgen\n\nAutomagically generate beautiful Adafruit-like pinout diagrams for IoT boards ✨\n\n## Introduction\n\nBoardgen allows to generate vector images, containing pinout diagrams for any IoT module. The images are inspired by Adafruit\'s great, well-recognized pinouts, and look like this one:\n\n![demo](.github/wr3-demo.png)\n\nIt is a Python tool that uses simple JSON files for defining shapes, colors, pin layouts and numbers. These files are then processed and drawn onto a .SVG image.\n\nDefining new boards is fully done in JSON and doesn\'t require Python programming, nor SVG knowledge.\n\n## Usage\n\nBoardgen can be used as a library (see [cli.py](boardgen/cli.py) for an example) or with its built-in CLI program.\n\n1. `pip install boardgen`\n2. `boardgen --help`\n3. `boardgen list boards` to get a list of available boards\n4. `boardgen draw <board name>` to produce an image file in your working directory\n\n```\nUsage: boardgen [OPTIONS] COMMAND [ARGS]...\n\n  boardgen CLI v0.1.0\n\nOptions:\n  --boards TEXT     Custom boards directories\n  --shapes TEXT     Custom shapes directories\n  --templates TEXT  Custom templates directories\n  --presets TEXT    Custom presets .json\n  --roles TEXT      Custom roles .json\n  --flash TEXT      Custom flash regions .json\n  --help            Show this message and exit.\n\nCommands:\n  draw  Draw board diagrams\n  list  List boards/templates/etc\n```\n\n### Board definitions\n\nWriting board definitions means putting a .JSON file to a directory and specifying this directory using the `--boards` option.\n\nBoardgen is meant to be used with PlatformIO-style board definition files. An example of such a board can be found [here (libretuya/boards/wr3.json)](https://github.com/kuba2k2/libretuya/blob/master/boards/wr3.json).\n\nNote that the board manifest uses `"_base"` definitions. These are merged recursively with the manifest, and this result is expected to produce a complete file.\n\nApart from PlatformIO default variables (such as `build`, `debug`, `upload` or `name`, `url` and `vendor`) the board definition contains a [`Pcb`](boardgen/models/pcb.py) object.\n\n### Templates\n\nA [`Template`](boardgen/models/template.py) is a JSON object containing lists of shapes for each side of a PCB. Additionally, it contains a mapping of pin names to shape IDs to allow automatic alignment of pinout labels.\n\n```json\n{\n\t"name": "demo-template",\n\t"title": "Demo template",\n\t"width": 10,\n\t"height": 20,\n\t"front": [\n\t\t{"info": "List of front side shapes goes here"},\n\t\t{\n\t\t\t"comment": "this is a sample rectangle",\n\t\t\t"id": "shape1",\n\t\t\t"type": "rect",\n\t\t\t"pos": "0,0",\n\t\t\t"size": "10,20",\n\t\t\t"fill": {"color": "black"}\n\t\t}\n\t],\n\t"back": [],\n\t"pads": {\n\t\t"1": "demo-template.front.shape1",\n\t\t"2": "demo-template.front.shape2",\n\t\t"3": "demo-template.front.shape3",\n\t\t"4": "demo-template.front.shape4",\n\t}\n}\n```\n\n### Shapes\n\nA shape JSON file is an array of multiple shape objects. Including shape JSON files is possible. For example, to include `my-shape.json`:\n\n```json\n{\n\t"name": "my-shape",\n\t"id": "id-of-my-shape",\n\t"pos": "5,10"\n}\n```\n\nEach shape is a JSON object, which has a common set of attributes:\n- `pos` - REQUIRED: defines the anchor of a shape (in milimeters)\n- `preset` (or a `presets` array) - used to merge the shape object with a [preset object](boardgen/res/presets.json) (see "boardgen list presets")\n- `id` - used to refer to the shape\'s anchor from the `pads` mapping\n- `vars` - mapping of variables for `${VAR}` substitution, optional\n\n#### Rectangles\n\n- `type`: must be `"rect"`\n- `pos`: REQUIRED: rectangle top-left corner position\n- `size`: REQUIRED: rectangle size ("width,height") in milimeters\n- `fill`: see FillStyle below\n- `stroke`: see FillStyle below\n- `rx`: X corner radius\n- `ry`: Y corner radius\n\n#### Circles\n\n- `type`: must be `"circle"`\n- `pos`: REQUIRED: circle center position\n- `d`: diameter (milimeters), OR:\n- `r`: radius (milimeters) - one of `d` and `r` is required\n- `fill`: see FillStyle below\n- `stroke`: see FillStyle below\n\n#### Text\n\n- `type`: must be `"text"`\n- `pos`: REQUIRED: text center position\n- `font_size`: REQUIRED: font size\n- `text`: REQUIRED: string to draw\n- `fill`: see FillStyle below\n\n#### FillStyle\n\n- `color`: string (color name, HTML hex), fill/stroke color\n- `lgrad`: linear gradient as a list ["x,y", "color", "x,y", "color"]\n  - 1st stop position\n  - 1st stop color\n  - 2nd stop position\n  - 2nd stop color\n- `width`: stroke width (not used for "fill")\n\n## Support\n\nAs this project is just a quick solution that\'ll be used in [LibreTuya](https://github.com/kuba2k2/libretuya) for generating pinouts, it\'s not really well documented. If you find this project interesting and need any help using it, feel free to open an issue and I\'ll try to provide more examples and info on how to use it.\n',
    'author': 'Kuba Szczodrzyński',
    'author_email': 'kuba@szczodrzynski.pl',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



# Copyright (c) Kuba Szczodrzyński 2022-06-15.

from abc import ABC

from .features import PinFeatures
from .section import SectionType

SectionItem = tuple[str, object, str]


class VariantParts(ABC):
    pins: dict[str, tuple[str, PinFeatures, str]] = {}
    sections: dict[SectionType, list[SectionItem]] = {}
    sorted_pins: list[tuple[str, str, PinFeatures, str]] = []
    sorted_sections: list[tuple[SectionType, list[SectionItem]]] = []

    def add_pin(self, name: str, gpio: str, comment: str = None) -> bool:
        if name in self.pins:
            return False
        self.pins[name] = (gpio, PinFeatures.PIN_NONE, comment)
        return True

    def add_pin_feature(self, name: str, feature: PinFeatures):
        if name not in self.pins:
            return
        (gpio, features, comment) = self.pins[name]
        features &= ~(PinFeatures.PIN_NONE)
        features |= feature
        self.pins[name] = (gpio, features, comment)

    def add_item(
        self, section: SectionType, key: str, value: object, comment: str = None
    ):
        if section not in self.sections:
            self.sections[section] = []
        self.sections[section].append((key, value, comment))

    def increment_item(self, section: SectionType, key: str):
        if section not in self.sections:
            self.sections[section] = []
        for i, (ikey, value, comment) in enumerate(self.sections[section]):
            if ikey != key:
                continue
            self.sections[section][i] = (key, value + 1, comment)
            return
        self.sections[section].append((key, 1, None))

    def format_pins(self) -> str:
        out = [
            "// clang-format off",
            "PinInfo pinTable[PINS_COUNT] = {",
        ]
        items: list[tuple[str, str, str, str]] = []
        max_gpio = 0
        max_features = 0
        for name, gpio, features, comment in self.sorted_pins:
            features_str = []
            for feature in PinFeatures:
                if features & feature:
                    features_str.append(feature.name)
            features_str = " | ".join(features_str)
            items.append((name, gpio, features_str, comment))
            max_gpio = max(max_gpio, len(gpio))
            max_features = max(max_features, len(features_str))
        for name, gpio, features, comment in items:
            out.append(f"\t// {name}: {comment}")
            pad_gpio = " " * (max_gpio - len(gpio))
            pad_features = " " * (max_features - len(features))
            out.append(
                f"\t{{{gpio}, {pad_gpio}{features}, {pad_features}PIN_NONE, 0}},"
            )
        out.append("};")
        out.append("// clang-format on")
        return "\n".join(out)

    def format_sections(self) -> str:
        out = [
            "// clang-format off",
            "",
        ]
        for section_type, items in self.sorted_sections:
            out.append(f"// {section_type.value}")
            out.append(f"// {len(section_type.value) * '-'}")
            max_key = 0
            max_value = 0
            for key, value, _ in items:
                value = str(value)
                max_key = max(max_key, len(key))
                max_value = max(max_value, len(value))
            for key, value, comment in items:
                value = str(value)
                if comment:
                    comment = f"// {comment}"
                else:
                    comment = ""
                pad_key = " " * (max_key - len(key))
                pad_value = " " * (max_value - len(value))
                out.append(
                    f"#define {key}{pad_key} {value}{pad_value} {comment}".strip()
                )
            out.append("")
        return "\n".join(out)
